#include <code/unicode/decoding.hxx>

#include <code/validation/main.hxx>

#include <string>

VALIDATION_TEST(test_1)
{
  std::string const encoded_0{ "\x00", 1 };
  std::string const encoded_1{ "\x7f", 1 };

  code::unicode::Utf8_decoder utf8;

  auto decoded_0 = utf8.decode(encoded_0.begin(), encoded_0.end());
  auto decoded_1 = utf8.decode(encoded_1.begin(), encoded_1.end());

  VALIDATION_ASSERT_EQUAL(decoded_0, 0U);
  VALIDATION_ASSERT_EQUAL(decoded_1, 0x7fU);
}

VALIDATION_TEST(test_2)
{
  std::string const encoded_0{ "\xc2\x80" };
  std::string const encoded_1{ "\xdf\xbf" };

  code::unicode::Utf8_decoder utf8;

  auto decoded_0 = utf8.decode(encoded_0.begin(), encoded_0.end());
  auto decoded_1 = utf8.decode(encoded_1.begin(), encoded_1.end());

  VALIDATION_ASSERT_EQUAL(decoded_0, 0x80U);
  VALIDATION_ASSERT_EQUAL(decoded_1, 0x7ffU);
}

VALIDATION_TEST(test_3)
{
  std::string const encoded_0{ "\xe0\xa0\x80" };
  std::string const encoded_1{ "\xef\xbf\xbf" };

  code::unicode::Utf8_decoder utf8;

  auto decoded_0 = utf8.decode(encoded_0.begin(), encoded_0.end());
  auto decoded_1 = utf8.decode(encoded_1.begin(), encoded_1.end());

  VALIDATION_ASSERT_EQUAL(decoded_0, 0x800U);
  VALIDATION_ASSERT_EQUAL(decoded_1, 0xffffU);
}

VALIDATION_TEST(test_4)
{
  std::string const encoded_0{ "\xf0\x90\x80\x80" };
  std::string const encoded_1{ "\xf4\x8f\xbf\xbf" };

  code::unicode::Utf8_decoder utf8;

  auto decoded_0 = utf8.decode(encoded_0.begin(), encoded_0.end());
  auto decoded_1 = utf8.decode(encoded_1.begin(), encoded_1.end());

  VALIDATION_ASSERT_EQUAL(decoded_0, 0x10000U);
  VALIDATION_ASSERT_EQUAL(decoded_1, 0x10ffffU);
}

int
main(int argc, char* argv[])
{
  return code::validation::main(argc, argv);
}
