#include <code/unicode/encoding.hxx>

#include <code/validation/main.hxx>

#include <sstream>
#include <string>

VALIDATION_TEST(test_1)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 1U);
  VALIDATION_ASSERT_EQUAL(string[0], 0);
}

VALIDATION_TEST(test_2)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0x7f);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 1U);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[0], 0x7f);
}

VALIDATION_TEST(test_3)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0x80);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 2U);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[0], 0xc2);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[1], 0x80);
}

VALIDATION_TEST(test_4)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0x7ff);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 2U);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[0], 0xdf);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[1], 0xbf);
}

VALIDATION_TEST(test_5)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0x800);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 3U);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[0], 0xe0);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[1], 0xa0);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[2], 0x80);
}

VALIDATION_TEST(test_6)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0xffff);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 3U);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[0], 0xef);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[1], 0xbf);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[2], 0xbf);
}

VALIDATION_TEST(test_7)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0x10000);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 4U);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[0], 0xf0);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[1], 0x90);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[2], 0x80);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[3], 0x80);
}

VALIDATION_TEST(test_8)
{
  std::stringstream str;

  code::unicode::Utf8_encoder utf8;
  utf8.encode(str, 0x10ffff);

  auto string = str.str();

  VALIDATION_ASSERT_EQUAL(string.size(), 4U);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[0], 0xf4);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[1], 0x8f);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[2], 0xbf);
  VALIDATION_ASSERT_EQUAL((unsigned char)string[3], 0xbf);
}

int
main(int argc, char* argv[])
{
  return code::validation::main(argc, argv);
}
